searchNodes=[{"doc":"Validates CSS files. It expects a file or a list of files as an argument. mix vnu.validate.css [OPTIONS] FILE [FILE2 FILE3...] See mix vnu.validate.html for the list of options and other details. Examples mix vnu.validate.css screen.css print.css mix vnu.validate.css --server-url localhost:8888 priv/static/**/*.css","ref":"Mix.Tasks.Vnu.Validate.Css.html","title":"Mix.Tasks.Vnu.Validate.Css","type":"task"},{"doc":"Validates HTML files. It expects a file or a list of files as an argument. mix vnu.validate.html [OPTIONS] FILE [FILE2 FILE3...] Options --server-url - The URL of the Checker server. Defaults to http://localhost:8888. --fail-on-warnings - Messages of type :info and subtype :warning will be treated as if they were validation errors. Their presence will mean the document is invalid. Defaults to false. --filter - A module implementing the Vnu.MessageFilter behavior that will be used to exclude messages matching the filter from the result. Defaults to nil (no excluded messages). Exit code If validation of all files finished and no errors were found, the task will exit with code 0. If the validation of at least one file did not finish for any reason, or resulted in validation errors, the task will exit with code 1. Examples mix vnu.validate.html index.html about.html mix vnu.validate.html --server-url localhost:8888 priv/static/**/*.html mix vnu.validate.html --filter MyApp.VnuMessageFilter priv/static/**/*.html","ref":"Mix.Tasks.Vnu.Validate.Html.html","title":"Mix.Tasks.Vnu.Validate.Html","type":"task"},{"doc":"Validates SVG files. It expects a file or a list of files as an argument. mix vnu.validate.svg [OPTIONS] FILE [FILE2 FILE3...] See mix vnu.validate.html for the list of options and other details. Examples mix vnu.validate.svg home_icon.svg logo.svg mix vnu.validate.svg --server-url localhost:8888 priv/static/**/*.svg","ref":"Mix.Tasks.Vnu.Validate.Svg.html","title":"Mix.Tasks.Vnu.Validate.Svg","type":"task"},{"doc":"General purpose validation functions for HTML, CSS, and SVG documents.","ref":"Vnu.html","title":"Vnu","type":"module"},{"doc":"Checks if the results of Vnu.validate_html/2, Vnu.validate_css/2, or Vnu.validate_svg/2 determined the document to be valid. Options :server_url - The URL of the Checker server. Defaults to http://localhost:8888. :fail_on_warnings - Messages of type :info and subtype :warning will be treated as if they were validation errors. Their presence will mean the document is invalid. Defaults to false. Examples iex&gt; {:ok, result} = Vnu.validate_html(&quot;&quot;, server_url: System.get_env(&quot;VNU_SERVER_URL&quot;) || &quot;http://localhost:8888&quot;) iex&gt; Vnu.valid?(result) false iex&gt; {:ok, result} = Vnu.validate_html(~S( ...&gt;&lt;!DOCTYPE html&gt; ...&gt;&lt;html&gt; ...&gt;&lt;head&gt; ...&gt; &lt;meta charset=&quot;utf-8&quot;&gt; ...&gt; &lt;title&gt;Hello World&lt;/title&gt; ...&gt;&lt;/head&gt; ...&gt;&lt;body&gt; ...&gt;&lt;/body&gt; ...&gt;&lt;/html&gt; ...&gt;), server_url: System.get_env(&quot;VNU_SERVER_URL&quot;) || &quot;http://localhost:8888&quot;) iex&gt; [message] = result.messages iex&gt; message.message &quot;Consider adding a “lang” attribute to the “html” start tag to declare the language of this document.&quot; iex&gt; message.sub_type :warning iex&gt; Vnu.valid?(result) true iex&gt; Vnu.valid?(result, fail_on_warnings: true) false","ref":"Vnu.html#valid?/2","title":"Vnu.valid?/2","type":"function"},{"doc":"Validates the given CSS document. See validate_html/2 for the list of options and other details. Examples iex&gt; Vnu.validate_css(&quot;.button { display: banana; }&quot;, server_url: System.get_env(&quot;VNU_SERVER_URL&quot;) || &quot;http://localhost:8888&quot;) {:ok, %Vnu.Result{messages: [ %Vnu.Message{ type: :error, message: &quot;“display”: “banana” is not a “display” value.&quot;, extract: &quot;.button { display: banana; }\\n&quot;, first_line: 1, last_line: 1, first_column: 20, last_column: 25, hilite_length: 6, hilite_start: 19, } ]}} iex&gt; Vnu.validate_css(&quot;&quot;, server_url: &quot;http://wrong-domain&quot;) {:error, %Vnu.Error{ reason: :unexpected_server_response, message: &quot;Could not contact the server, got error: %HTTPoison.Error{id: nil, reason: :nxdomain}&quot; }}","ref":"Vnu.html#validate_css/2","title":"Vnu.validate_css/2","type":"function"},{"doc":"Same as validate_css/2 but returns %Vnu.Result{} or raises %Vnu.Error{}.","ref":"Vnu.html#validate_css!/2","title":"Vnu.validate_css!/2","type":"function"},{"doc":"Validates the given HTML document. Returns {:ok, %Vnu.Result{}} if the validation process finished successfully, and {:error, %Vnu.Error{}} otherwise. Note that the {:ok, %Vnu.Result{}} return value does not mean necessarily that the document is valid. See Vnu.valid?/1 and Vnu.Message for interpreting the result. Options :server_url - The URL of the Checker server. Defaults to http://localhost:8888. :filter - A module implementing the Vnu.MessageFilter behavior that will be used to exclude messages matching the filter from the result. Defaults to nil (no excluded messages). Examples iex&gt; Vnu.validate_html(~S( ...&gt;&lt;!DOCTYPE html&gt; ...&gt;&lt;html&gt; ...&gt;&lt;head&gt; ...&gt; &lt;meta charset=&quot;utf-8&quot;&gt; ...&gt;&lt;/head&gt; ...&gt;&lt;body&gt; ...&gt;&lt;/body&gt; ...&gt;&lt;/html&gt; ...&gt;), server_url: System.get_env(&quot;VNU_SERVER_URL&quot;) || &quot;http://localhost:8888&quot;) {:ok, %Vnu.Result{messages: [ %Vnu.Message{ type: :error, message: &quot;Element “head” is missing a required instance of child element “title”.&quot;, extract: &quot;=\\&quot;utf-8\\&quot;&gt;\\n&lt;/head&gt;\\n&lt;body&quot;, first_line: 6, last_line: 6, first_column: 1, last_column: 7, hilite_length: 7, hilite_start: 10 }, %Vnu.Message{ type: :info, sub_type: :warning, message: &quot;Consider adding a “lang” attribute to the “html” start tag to declare the language of this document.&quot;, extract: &quot;TYPE html&gt;\\n&lt;html&gt;\\n&lt;head&quot;, first_line: 2, last_line: 3, first_column: 16, last_column: 6, hilite_length: 7, hilite_start: 10, } ]}} iex&gt; Vnu.validate_html(&quot;&quot;, server_url: &quot;http://wrong-domain&quot;) {:error, %Vnu.Error{ reason: :unexpected_server_response, message: &quot;Could not contact the server, got error: %HTTPoison.Error{id: nil, reason: :nxdomain}&quot; }}","ref":"Vnu.html#validate_html/2","title":"Vnu.validate_html/2","type":"function"},{"doc":"Same as validate_svg/2 but returns %Vnu.Result{} or raises %Vnu.Error{}.","ref":"Vnu.html#validate_html!/2","title":"Vnu.validate_html!/2","type":"function"},{"doc":"Validates the given SVG document. See validate_html/2 for the list of options and other details. Options :server_url - The URL of the Checker server. Defaults to http://localhost:8888. Examples iex&gt; Vnu.validate_svg(~S( ...&gt;&lt;svg width=&quot;5cm&quot; height=&quot;4cm&quot; version=&quot;1.1&quot; xmlns=&quot;http://www.w3.org/2000/svg&quot;&gt; ...&gt;&lt;desc&gt;Rectangle&lt;/desc&gt; ...&gt;&lt;rect x=&quot;0.5cm&quot; y=&quot;0.5cm&quot; height=&quot;1cm&quot;/&gt; ...&gt;&lt;/svg&gt; ...&gt; ), server_url: System.get_env(&quot;VNU_SERVER_URL&quot;) || &quot;http://localhost:8888&quot;) {:ok, %Vnu.Result{messages: [ %Vnu.Message{ type: :info, message: &quot;Using the preset for SVG 1.1 + URL + HTML + MathML 3.0 based on the root namespace.&quot; }, %Vnu.Message{ type: :error, message: &quot;SVG element “rect” is missing required attribute “width”.&quot;, extract: &quot;le&lt;/desc&gt;\\n&lt;rect x=\\&quot;0.5cm\\&quot; y=\\&quot;0.5cm\\&quot; height=\\&quot;1cm\\&quot;/&gt;\\n&lt;/svg&quot;, first_line: 4, last_line: 4, first_column: 1, last_column: 40, hilite_length: 40, hilite_start: 10, } ]}} iex&gt; Vnu.validate_svg(&quot;&quot;, server_url: &quot;http://wrong-domain&quot;) {:error, %Vnu.Error{ reason: :unexpected_server_response, message: &quot;Could not contact the server, got error: %HTTPoison.Error{id: nil, reason: :nxdomain}&quot; }}","ref":"Vnu.html#validate_svg/2","title":"Vnu.validate_svg/2","type":"function"},{"doc":"Same as validate_svg/2 but returns %Vnu.Result{} or raises %Vnu.Error{}.","ref":"Vnu.html#validate_svg!/2","title":"Vnu.validate_svg!/2","type":"function"},{"doc":"ExUnit assertions for checking the validity of HTML, CSS, and SVG documents.","ref":"Vnu.Assertions.html","title":"Vnu.Assertions","type":"module"},{"doc":"Asserts that the given CSS document is valid. See assert_valid_html/2 for the list of options and other details.","ref":"Vnu.Assertions.html#assert_valid_css/2","title":"Vnu.Assertions.assert_valid_css/2","type":"function"},{"doc":"Asserts that the given HTML document is valid. Options :server_url - The URL of the Checker server. Defaults to http://localhost:8888. :fail_on_warnings - Messages of type :info and subtype :warning will be treated as if they were validation errors. Their presence will mean the document is invalid. Defaults to false. :filter - A module implementing the Vnu.MessageFilter behavior that will be used to exclude messages matching the filter from the result. Defaults to nil (no excluded messages). :message_print_limit - The maximum number of validation messages that will me printed in the error when the assertion fails. Can be an integer or :infinity. Defaults to :infinity.","ref":"Vnu.Assertions.html#assert_valid_html/2","title":"Vnu.Assertions.assert_valid_html/2","type":"function"},{"doc":"Asserts that the given SVG document is valid. See assert_valid_html/2 for the list of options and other details.","ref":"Vnu.Assertions.html#assert_valid_svg/2","title":"Vnu.Assertions.assert_valid_svg/2","type":"function"},{"doc":"A message is a unit of information returned by the Checker. See its documentation for detailed up to date information about its output format. Fields :type - One of :error, :info, or :non_document_error. Info messages can either be general information or warnings, see :sub_type. Non-document errors signify errors with the Checker server itself, and are treated internally by this library as if the validation could not be run at all. :sub_type - For messages of type :error it could be nil or :fatal. For messages of type :info, it could be nil or :warning. :message - The detailed description of the issue. :extract - The snippet of the document that the message is about. :first_line, :last_line, :first_column, :last_column - The position of the part of the document the message is about relative to the whole document. Lines and columns are numbered from 1. :hilite_start, :hilite_length - Indicate the start and length of substring of the :extract that the message is roughly about. The characters are numbered from 0.","ref":"Vnu.Message.html","title":"Vnu.Message","type":"module"},{"doc":"","ref":"Vnu.Message.html#t:t/0","title":"Vnu.Message.t/0","type":"type"},{"doc":"A behavior for excluding certain messages from the validation result. Modules implementing this behavior can be passed as a :filter option to: Vnu.validate_html/2, Vnu.validate_css/2, Vnu.validate_svg/2 Vnu.Assertions.assert_valid_html/2, Vnu.Assertions.assert_valid_css/2. Vnu.Assertions.assert_valid_svg/2 Example defmodule MyApp.VnuMessageFilter do @behaviour Vnu.MessageFilter @impl Vnu.MessageFilter def exclude_message?(%Vnu.Message{message: message}) do # those errors are caused by the CSRF meta tag (`csrf_meta_tag()`) # present in the layout of a newly-generated Phoenix app patterns_to_ignore = [ ~r/A document must not include more than one “meta” element with a “charset” attribute./, ~r/Attribute “(.)*” not allowed on element “meta” at this point./ ] Enum.any?(patterns_to_ignore, &amp;Regex.match?(&amp;1, message)) end end","ref":"Vnu.MessageFilter.html","title":"Vnu.MessageFilter","type":"behaviour"},{"doc":"","ref":"Vnu.MessageFilter.html#c:exclude_message?/1","title":"Vnu.MessageFilter.exclude_message?/1","type":"callback"},{"doc":"A result holds a list of messages returned by the Checker for a given document.","ref":"Vnu.Result.html","title":"Vnu.Result","type":"module"},{"doc":"","ref":"Vnu.Result.html#t:t/0","title":"Vnu.Result.t/0","type":"type"},{"doc":"An error holds details about why validating a given document could not be finished.","ref":"Vnu.Error.html","title":"Vnu.Error","type":"exception"},{"doc":"","ref":"Vnu.Error.html#t:t/0","title":"Vnu.Error.t/0","type":"type"},{"doc":"Vnu An Elixir client for the Nu HTML Checker (v.Nu). v.Nu is a document validity checker used by the W3C. It offers validating HTML, CSS, and SVG documents. This library brings that functionality to Elixir by using the Checker&#39;s JSON API. It offers ExUnit assertions for validating dynamic content in tests, Mix tasks for validating static content, and general purpose functions to fulfill other needs.","ref":"readme.html","title":"Vnu","type":"extras"},{"doc":"While it is possible to use this library with the service run by W3C at validator.w3.org/nu, I would recommend running your own instance. You will eliminate a lot of network latency if it runs on the same machine as your code, and you will not hit any rate limits that might exist for validator.w3.org/nu. The source of the Checker can be found in the repository validator/validator. Follow their instructions on how to download it and run it as a web server. The easiest option is to use the Docker image, like this: docker run -it --rm -p 8888:8888 validator/validator:latest Check if the server is running: $ curl localhost:8888 -I HTTP/1.1 200 OK","ref":"readme.html#prerequisites","title":"Vnu - Prerequisites","type":"extras"},{"doc":"Make sure to read about the prerequisites first. Add Vnu as a dependency to your project&#39;s mix.exs: defp deps do [ {:vnu, &quot;~&gt; 1.0&quot;, only: [:dev, :test], runtime: false} ] end And run: $ mix deps.get","ref":"readme.html#installation","title":"Vnu - Installation","type":"extras"},{"doc":"Available on hexdocs.pm.","ref":"readme.html#documentation","title":"Vnu - Documentation","type":"extras"},{"doc":"ExUnit assertions If you are building an application that generates HTML, CSS, or SVG files, you might want to use those validations in your tests. Vnu.Assertions.assert_valid_html/2 Vnu.Assertions.assert_valid_css/2 Vnu.Assertions.assert_valid_svg/2 Phoenix controller test example defmodule PhoenixAppWeb.PageControllerTest do use PhoenixAppWeb.ConnCase import Vnu.Assertions test &quot;GET /&quot;, %{conn: conn} do vnu_opts = %{server_url: &quot;http://localhost:8888&quot;, fail_on_warnings: true} conn = get(conn, &quot;/&quot;) html_response = conn |&gt; get(&quot;/&quot;) |&gt; html_response(200) |&gt; assert_valid_html(vnu_opts) assert html_response =~ &quot;Welcome to Phoenix!&quot; end end See examples/1_phoenix_app/test/phoenix_app_web/controllers/page_controller_test.exs for more. Mix task If you have static HTML, CSS, or SVG files in your project, you might want to validate them with those mix tasks: mix vnu.validate.html mix vnu.validate.css mix vnu.validate.svg Example $ mix vnu.validate.css --server-url localhost:8888 assets/**/*.css General purpose If you need HTML, CSS, or SVG validation for something else, try one of those functions: Vnu.validate_html/2 Vnu.validate_css/2 Vnu.validate_svg/2 iex&gt; {:ok, result} = Vnu.validate_html(&quot;&lt;!DOCTYPE html&gt;&lt;html&gt;&lt;head&gt;&lt;/head&gt;&lt;/html&gt;&quot;, server_url: &quot;http://localhost:8888&quot;) {:ok, %Vnu.Result{ messages: [ %Vnu.Message{ extract: &quot;tml&gt;&lt;head&gt;&lt;/head&gt;&lt;/html&quot;, first_column: 28, first_line: 1, hilite_length: 7, hilite_start: 10, last_column: 34, last_line: 1, message: &quot;Element “head” is missing a required instance of child element “title”.&quot;, offset: nil, sub_type: nil, type: :error }, %Vnu.Message{ extract: &quot;TYPE html&gt;&lt;html&gt;&lt;head&gt;&quot;, first_column: 16, first_line: 1, hilite_length: 6, hilite_start: 10, last_column: 21, last_line: 1, message: &quot;Consider adding a “lang” attribute to the “html” start tag to declare the language of this document.&quot;, offset: nil, sub_type: :warning, type: :info } ] }} iex&gt; Vnu.valid?(result) false","ref":"readme.html#usage","title":"Vnu - Usage","type":"extras"},{"doc":"Make sure to read about the prerequisites first. After cloning the repository, run mix deps.get and you should be ready for development. To ensure code consistency, run mix format, mix credo, and mix dialyzer. Running tests All test that expect to talk with the server accept the server&#39;s URL as an VNU_SERVER_URL environment variable or fallback to the default http://localhost:8888. $ VNU_SERVER_URL=http://localhost:4000/ mix test If you&#39;re adding a new test, make sure it will do that do.","ref":"readme.html#development","title":"Vnu - Development","type":"extras"},{"doc":"Issues If you noticed a problem with the library or its documentation, or have an idea for a feature, open an issue. If you have an idea on how to act upon the problem or idea, feel free to open a pull request instead. Pull requests If you noticed a problem with the library or its documentation and know how to fix it, or have an idea for a feature, or want to fix a typo, open a pull request. If you are not sure of your changes or need help finishing them, open a pull request anyway. I&#39;ll try to help!","ref":"readme.html#contributing","title":"Vnu - Contributing","type":"extras"},{"doc":"Vnu is released under the MIT License. See the LICENSE file for further details.","ref":"readme.html#license","title":"Vnu - License","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Changed When used incorrectly, instead of custom usage info, mix tasks will print the same message as running mix help ... would print Lowered required httpoison and jason versions to ~&gt; 1.0","ref":"changelog.html#1-0-0-2020-04-13","title":"Changelog - 1.0.0 (2020-04-13)","type":"extras"},{"doc":"Initial release candidate","ref":"changelog.html#1-0-0-rc-1-2020-04-11","title":"Changelog - 1.0.0-rc.1 (2020-04-11)","type":"extras"}]